@@ -387,6 +387,10 @@ class StaggeredGrid(Grid):
         item = slicing_dict(self, item)
         if not item:
             return self
+        if 'vector' in item:
+            selection = item['vector']
+            if isinstance(selection, int):
+                item['vector'] = self.resolution.names[selection]
         values = self._values[{dim: sel for dim, sel in item.items() if dim not in self.shape.spatial}]
         for dim, sel in item.items():
             if dim in self.shape.spatial:
@@ -410,7 +414,7 @@ class StaggeredGrid(Grid):
                 assert selection in item_names, f"Accessing field.vector['{selection}'] failed. Item names are {item_names}."
                 selection = item_names.index(selection)
             if isinstance(selection, int):
-                dim = self.shape.spatial.names[selection]
+                dim = self.resolution.names[selection]
                 comp_cells = GridCell(self.resolution, bounds).stagger(dim, *self.extrapolation.valid_outer_faces(dim))
                 return CenteredGrid(values, bounds=comp_cells.bounds, extrapolation=extrapolation)
             else:
@@ -437,7 +441,7 @@ class StaggeredGrid(Grid):
             widths = {d: (0, 1) for d in self.resolution.names}
             lo_valid, up_valid = self.extrapolation.valid_outer_faces(dim)
             widths[dim] = (int(not lo_valid), int(not up_valid))
-            padded.append(math.pad(component, widths, self.extrapolation, bounds=self.bounds))
+            padded.append(math.pad(component, widths, self.extrapolation[{'vector': dim}], bounds=self.bounds))
         result = math.stack(padded, channel(vector=self.resolution))
         assert result.shape.is_uniform
         return result
