@@ -5,7 +5,7 @@ import numpy as np
 from . import _ops as math
 from . import extrapolation as extrapolation
 from ._magic_ops import stack, rename_dims, concat, variable_values
-from ._shape import Shape, channel, batch, spatial, DimFilter, parse_dim_order
+from ._shape import Shape, channel, batch, spatial, DimFilter, parse_dim_order, shape
 from ._tensors import Tensor, wrap
 from .magic import PhiTreeNode
 from .extrapolation import Extrapolation
@@ -146,7 +146,7 @@ def dim_mask(all_dims: Shape or tuple or list, dims: DimFilter, mask_dim=channel
         all_dims = spatial(*all_dims)
     dims = all_dims.only(dims)
     mask = [1 if dim in dims else 0 for dim in all_dims]
-    mask_dim = mask_dim._with_item_names((all_dims.names,))
+    mask_dim = mask_dim.with_size(all_dims.names)
     return wrap(mask, mask_dim)
 
 
@@ -319,7 +319,8 @@ def shift(x: Tensor,
           offsets: tuple,
           dims: DimFilter = math.spatial,
           padding: Extrapolation or None = extrapolation.BOUNDARY,
-          stack_dim: Optional[Shape] = channel('shift')) -> list:
+          stack_dim: Optional[Shape] = channel('shift'),
+          extend_bounds=0) -> list:
     """
     shift Tensor by a fixed offset and abiding by extrapolation
 
@@ -343,7 +344,9 @@ def shift(x: Tensor,
     pad_lower = max(0, -min(offsets))
     pad_upper = max(0, max(offsets))
     if padding:
-        x = math.pad(x, {axis: (pad_lower, pad_upper) for axis in dims}, mode=padding)
+        x = math.pad(x, {axis: (pad_lower + extend_bounds, pad_upper + extend_bounds) for axis in dims}, mode=padding)
+    if extend_bounds:
+        assert padding is not None
     offset_tensors = []
     for offset in offsets:
         components = []
@@ -435,7 +438,8 @@ def spatial_gradient(grid: Tensor,
                      difference: str = 'central',
                      padding: Extrapolation or None = extrapolation.BOUNDARY,
                      dims: DimFilter = spatial,
-                     stack_dim: Shape or None = channel('gradient')) -> Tensor:
+                     stack_dim: Shape or None = channel('gradient'),
+                     pad=0) -> Tensor:
     """
     Calculates the spatial_gradient of a scalar channel from finite differences.
     The spatial_gradient vectors are in reverse order, lowest dimension first.
@@ -448,6 +452,8 @@ def spatial_gradient(grid: Tensor,
         difference: type of difference, one of ('forward', 'backward', 'central') (default 'forward')
         padding: tensor padding mode
         stack_dim: name of the new vector dimension listing the spatial_gradient w.r.t. the various axes
+        pad: How many cells to extend the result compared to `grid`.
+            This value is added to the internal padding. For non-trivial extrapolations, this gives the correct result while manual padding before or after this operation would not respect the boundary locations.
 
     Returns:
         `Tensor`
@@ -463,13 +469,13 @@ def spatial_gradient(grid: Tensor,
         if dx.vector.size in (None, 1):
             dx = dx.vector[0]
     if difference.lower() == 'central':
-        left, right = shift(grid, (-1, 1), dims, padding, stack_dim=stack_dim)
+        left, right = shift(grid, (-1, 1), dims, padding, stack_dim=stack_dim, extend_bounds=pad)
         return (right - left) / (dx * 2)
     elif difference.lower() == 'forward':
-        left, right = shift(grid, (0, 1), dims, padding, stack_dim=stack_dim)
+        left, right = shift(grid, (0, 1), dims, padding, stack_dim=stack_dim, extend_bounds=pad)
         return (right - left) / dx
     elif difference.lower() == 'backward':
-        left, right = shift(grid, (-1, 0), dims, padding, stack_dim=stack_dim)
+        left, right = shift(grid, (-1, 0), dims, padding, stack_dim=stack_dim, extend_bounds=pad)
         return (right - left) / dx
     else:
         raise ValueError('Invalid difference type: {}. Can be CENTRAL or FORWARD'.format(difference))
@@ -480,7 +486,8 @@ def spatial_gradient(grid: Tensor,
 def laplace(x: Tensor,
             dx: Tensor or float = 1,
             padding: Extrapolation = extrapolation.BOUNDARY,
-            dims: DimFilter = spatial):
+            dims: DimFilter = spatial,
+            weights: Tensor = None):
     """
     Spatial Laplace operator as defined for scalar fields.
     If a vector field is passed, the laplace is computed component-wise.
@@ -490,6 +497,8 @@ def laplace(x: Tensor,
         dx: scalar or 1d tensor
         padding: extrapolation
         dims: The second derivative along these dimensions is summed over
+        weights: (Optional) Multiply the axis terms by these factors before summation.
+            Must be a Tensor with a single channel dimension that lists all laplace dims by name.
 
     Returns:
         `phi.math.Tensor` of same shape as `x`
@@ -502,6 +511,11 @@ def laplace(x: Tensor,
         return x.spatial_gradient()
     left, center, right = shift(wrap(x), (-1, 0, 1), dims, padding, stack_dim=batch('_laplace'))
     result = (left + right - 2 * center) / (dx ** 2)
+    if weights is not None:
+        dim_names = x.shape.only(dims).names
+        assert channel(weights).rank == 1 and channel(weights).item_names is not None, f"weights must have one channel dimension listing the laplace dims but got {shape(weights)}"
+        assert set(channel(weights).item_names[0]) >= set(dim_names), f"the channel dim of weights must contain all laplace dims {dim_names} but only has {channel(weights).item_names}"
+        result *= rename_dims(weights, channel, batch('_laplace'))
     result = math.sum_(result, '_laplace')
     return result
 
