@@ -609,7 +609,7 @@ def meshgrid(dim_type=spatial, stack_dim=channel('vector'), assign_item_names=Tr
     grid_shape = dim_type(**{dim: size for dim, size in zip(dimensions.keys(), dim_sizes)})
     channels = [NativeTensor(t, grid_shape) for t in indices_list]
     if assign_item_names:
-        return stack_tensors(channels, stack_dim._with_item_names((tuple(dimensions.keys()),)))
+        return stack_tensors(channels, stack_dim.with_size(tuple(dimensions.keys())))
     else:
         return stack_tensors(channels, stack_dim)
 
@@ -633,6 +633,10 @@ def linspace(start: int or Tensor, stop, dim: Shape) -> Tensor:
     """
     assert isinstance(dim, Shape) and dim.rank == 1, f"dim must be a single-dimension Shape but got {dim}"
     if is_scalar(start) and is_scalar(stop):
+        if isinstance(start, Tensor):
+            start = start.native()
+        if isinstance(stop, Tensor):
+            stop = stop.native()
         native_linspace = choose_backend(start, stop, prefer_default=True).linspace(start, stop, dim.size)
         return NativeTensor(native_linspace, dim)
     else:
@@ -701,16 +705,23 @@ def stack_tensors(values: tuple or list, dim: Shape):
 def concat_tensor(values: tuple or list, dim: str) -> Tensor:
     assert len(values) > 0, "concat() got empty sequence"
     assert isinstance(dim, str), f"dim must be a single-dimension Shape but got '{dim}' of type {type(dim)}"
-    broadcast_shape = merge_shapes(*[t.shape._with_item_name(dim, None).with_sizes([None] * t.shape.rank) for t in values])
-    natives = [v.native(order=broadcast_shape.names) for v in values]
-    backend = choose_backend(*natives)
-    concatenated = backend.concat(natives, broadcast_shape.index(dim))
-    if all([v.shape.get_item_names(dim) is not None for v in values]):
-        broadcast_shape = broadcast_shape._with_item_name(dim, sum([v.shape.get_item_names(dim) for v in values], ()))
-    return NativeTensor(concatenated, broadcast_shape.with_sizes(backend.staticshape(concatenated)))
+
+    def inner_concat(*values):
+        broadcast_shape: Shape = values[0].shape  # merge_shapes(*[t.shape.with_sizes([None] * t.shape.rank) for t in values])
+        dim_index = broadcast_shape.index(dim)
+        natives = [v.native(order=broadcast_shape.names) for v in values]
+        concatenated = choose_backend(*natives).concat(natives, dim_index)
+        if all([v.shape.get_item_names(dim) is not None for v in values]):
+            broadcast_shape = broadcast_shape.with_dim_size(dim, sum([v.shape.get_item_names(dim) for v in values], ()))
+        else:
+            broadcast_shape = broadcast_shape.with_dim_size(dim, sum([v.shape.get_size(dim) for v in values]))
+        return NativeTensor(concatenated, broadcast_shape)
+
+    result = broadcast_op(inner_concat, values)
+    return result
 
 
-def pad(value: Tensor, widths: dict, mode: 'e_.Extrapolation', **kwargs) -> Tensor:
+def pad(value: Tensor, widths: dict, mode: 'e_.Extrapolation' or Tensor or Number, **kwargs) -> Tensor:
     """
     Pads a tensor along the specified dimensions, determining the added values using the given extrapolation.
     Unlike `Extrapolation.pad()`, this function can handle negative widths which slice off outer values.
@@ -720,6 +731,7 @@ def pad(value: Tensor, widths: dict, mode: 'e_.Extrapolation', **kwargs) -> Tens
         widths: `dict` mapping dimension name (`str`) to `(lower, upper)`
             where `lower` and `upper` are `int` that can be positive (pad), negative (slice) or zero (pass).
         mode: `Extrapolation` used to determine values added from positive `widths`.
+            Assumes constant extrapolation if given a number or `Tensor` instead.
         kwargs: Additional padding arguments.
             These are ignored by the standard extrapolations defined in `phi.math.extrapolation` but can be used to pass additional contextual information to custom extrapolations.
             Grid classes from `phi.field` will pass the argument `bounds: Box`.
@@ -727,6 +739,7 @@ def pad(value: Tensor, widths: dict, mode: 'e_.Extrapolation', **kwargs) -> Tens
     Returns:
         Padded `Tensor`
     """
+    mode = mode if isinstance(mode, e_.Extrapolation) else e_.ConstantExtrapolation(mode)
     has_negative_widths = any(w[0] < 0 or w[1] < 0 for w in widths.values())
     slices = None
     if has_negative_widths:
@@ -886,6 +899,7 @@ def broadcast_op(operation: Callable,
         dim = next(iter(iter_dims))
         dim_type = None
         size = None
+        item_names = None
         unstacked = []
         for tensor in tensors:
             if dim in tensor.shape.names:
@@ -897,6 +911,8 @@ def broadcast_op(operation: Callable,
                 else:
                     assert size == len(unstacked_tensor)
                     assert dim_type == tensor.shape.get_type(dim)
+                if item_names is None:
+                    item_names = tensor.shape.get_item_names(dim)
             else:
                 unstacked.append(tensor)
         result_unstacked = []
@@ -904,7 +920,7 @@ def broadcast_op(operation: Callable,
             gathered = [t[i] if isinstance(t, tuple) else t for t in unstacked]
             result_unstacked.append(broadcast_op(operation, gathered, iter_dims=set(iter_dims) - {dim}))
         if not no_return:
-            return TensorStack(result_unstacked, Shape((None,), (dim,), (dim_type,), (None,)))
+            return TensorStack(result_unstacked, Shape((None,), (dim,), (dim_type,), (item_names,)))
 
 
 def where(condition: Tensor or float or int, value_true: Tensor or float or int, value_false: Tensor or float or int):
@@ -1968,6 +1984,8 @@ def dtype(x) -> DType:
 
 
 def expand_tensor(value: float or Tensor, dims: Shape):
+    if not dims:
+        return value
     value = wrap(value)
     shape = value.shape
     for dim in reversed(dims):
@@ -1977,7 +1995,12 @@ def expand_tensor(value: float or Tensor, dims: Shape):
             if dim.size is None:
                 dim = dim.with_sizes([1])
             shape = concat_shapes(dim, shape)
-    return CollapsedTensor(value, shape)
+    if shape == value.shape:  # no changes made
+        return value
+    elif shape.rank == value.rank:  # changes made but same dims
+        return value._with_shape_replaced(shape._reorder(value.shape))
+    else:
+        return CollapsedTensor(value, shape)
 
 
 def close(*tensors, rel_tolerance=1e-5, abs_tolerance=0) -> bool:
