@@ -152,6 +152,9 @@ class Extrapolation:
     def __getitem__(self, item):
         return self
 
+    def _getitem_with_domain(self, item: dict, dim: str, upper_edge: bool, all_dims: tuple):
+        return self[item]
+
     def __abs__(self):
         raise NotImplementedError(self.__class__)
 
@@ -240,10 +243,11 @@ class ConstantExtrapolation(Extrapolation):
         if isinstance(value, NativeTensor):
             native = value._native
             ordered_pad_widths = order_by_shape(value.shape, widths, default=(0, 0))
-            backend = choose_backend(native)
+            backend = choose_backend(native, pad_value.native())
             result_tensor = backend.pad(native, ordered_pad_widths, 'constant', pad_value.native())
-            new_shape = value.shape.with_sizes(backend.staticshape(result_tensor))
-            return NativeTensor(result_tensor, new_shape)
+            if result_tensor is NotImplemented:
+                return Extrapolation.pad(self, value, widths, **kwargs)
+            return NativeTensor(result_tensor, value.shape.after_pad(widths))
         elif isinstance(value, CollapsedTensor):
             if value._inner.shape.volume > 1 or not math.all_available(pad_value, value) or not math.close(pad_value, value._inner):  # .inner should be safe after _simplify
                 return self.pad(value._cache(), widths)
@@ -255,8 +259,7 @@ class ConstantExtrapolation(Extrapolation):
                     else:
                         delta = sum(widths[dim]) if isinstance(widths[dim], (tuple, list)) else 2 * widths[dim]
                         new_sizes.append(size + int(delta))
-                new_shape = value.shape.with_sizes(new_sizes)
-                return CollapsedTensor(value._inner, new_shape)
+                return CollapsedTensor(value._inner, value.shape.after_pad(widths))
         elif isinstance(value, TensorStack):
             if not value.requires_broadcast:
                 return self.pad(value._cache(), widths)
@@ -382,24 +385,13 @@ class _CopyExtrapolation(Extrapolation):
             result_tensor = choose_backend(native).pad(native, ordered_pad_widths, repr(self))
             if result_tensor is NotImplemented:
                 return Extrapolation.pad(self, value, widths)
-            new_shape = value.shape.with_sizes(result_tensor.shape)
-            return NativeTensor(result_tensor, new_shape)
+            return NativeTensor(result_tensor, value.shape.after_pad(widths))
         elif isinstance(value, CollapsedTensor):
             inner = value._inner  # should be fine after _simplify
             inner_widths = {dim: w for dim, w in widths.items() if dim in inner.shape}
             if len(inner_widths) > 0:
                 inner = self.pad(inner, widths)
-            new_sizes = []
-            for size, dim, *_ in value.shape._dimensions:
-                if dim not in widths:
-                    new_sizes.append(size)
-                else:
-                    delta = sum(widths[dim]) if isinstance(widths[dim], (tuple, list)) else 2 * widths[dim]
-                    new_sizes.append(size + int(delta))
-            new_shape = value.shape.with_sizes(new_sizes)
-            return CollapsedTensor(inner, new_shape)
-        # elif isinstance(value, SparseLinearOperation):
-        #     return pad_operator(value, widths, mode)
+            return CollapsedTensor(inner, value.shape.after_pad(widths))
         elif isinstance(value, TensorStack):
             if not value.requires_broadcast:
                 return self.pad(value._cache(), widths)
@@ -902,7 +894,8 @@ class _MixedExtrapolation(Extrapolation):
 
     def __getitem__(self, item):
         if isinstance(item, dict):
-            return combine_sides(**{dim: (e1[item], e2[item]) for dim, (e1, e2) in self.ext.items()})
+            all_dims = tuple(self.ext.keys())
+            return combine_sides(**{dim: (e1._getitem_with_domain(item, dim, False, all_dims), e2._getitem_with_domain(item, dim, True, all_dims)) for dim, (e1, e2) in self.ext.items()})
         else:
             dim, face = item
             return self.ext[dim][face]
@@ -992,9 +985,22 @@ class _NormalTangentialExtrapolation(Extrapolation):
         result = stack(result, value.shape.only('vector'))
         return result
 
+    def _getitem_with_domain(self, item: dict, dim: str, upper_edge: bool, all_dims: tuple):
+        if 'vector' not in item:
+            return self
+        component = item['vector']
+        assert isinstance(component, str), f"Selecting a component of normal/tangential must be done by dimension name but got {component}"
+        if component == dim:
+            return self.normal
+        else:
+            return self.tangential
+
     def __eq__(self, other):
         return isinstance(other, _NormalTangentialExtrapolation) and self.normal == other.normal and self.tangential == other.tangential
 
+    def __hash__(self):
+        return hash(self.normal) + hash(self.tangential)
+
     def __add__(self, other):
         return self._op2(other, lambda e1, e2: e1 + e2)
 
