@@ -14,7 +14,7 @@ See `phi.flow`, `phi.torch.flow`, `phi.jax.flow`.
 
 from phi.flow import *
 from . import TENSORFLOW
-from .nets import parameter_count, dense_net, u_net, save_state, load_state, update_weights, adam
+from .nets import parameter_count, get_parameters, dense_net, u_net, save_state, load_state, update_weights, adam, conv_net, res_net, sgd, sgd as SGD, adagrad, rmsprop, conv_classifier
 import tensorflow
 from tensorflow import keras
 from tensorflow.keras import layers
@@ -26,4 +26,4 @@ tf = tensorflow
 if not backend.context_backend():
     backend.set_global_default_backend(TENSORFLOW)
 else:
-    _LOGGER.warn(f"Importing '{__name__}' within a backend context will not set the default backend.")
+    _LOGGER.warn(f"Importing '{__name__}' within a backend context will not set the default backend.")
\ No newline at end of file
